@@ -1,9 +1,12 @@
 """The main API for the ``wombat``."""
+from __future__ import annotations
 
-import attr
-import pandas as pd
 from typing import List, Union, Optional
 from pathlib import Path
+
+import yaml
+import pandas as pd
+from attrs import Attribute, field, define
 from simpy.events import Event  # type: ignore
 
 from wombat.core import (
@@ -35,7 +38,7 @@ def _library_mapper(file_path: Union[str, Path]) -> Path:
     return Path(library_map.get(file_path, file_path)).resolve()  # type: ignore
 
 
-@attr.s(frozen=True, auto_attribs=True)
+@define(frozen=True, auto_attribs=True)
 class Configuration(FromDictMixin):
     """The ``Simulation`` configuration data class that provides all the necessary definitions.
 
@@ -77,7 +80,7 @@ class Configuration(FromDictMixin):
     """
 
     name: str
-    library: Path = attr.ib(converter=_library_mapper)
+    library: Path = field(converter=_library_mapper)
     layout: str
     service_equipment: Union[str, List[str]]
     weather: Union[str, pd.DataFrame]
@@ -86,39 +89,46 @@ class Configuration(FromDictMixin):
     inflation_rate: float
     fixed_costs: str
     project_capacity: Union[int, float]
-    start_year: int = attr.ib(default=None)
-    end_year: int = attr.ib(default=None)
-    SAM_settings: str = attr.ib(default=None)
+    start_year: int = field(default=None)
+    end_year: int = field(default=None)
+    SAM_settings: str = field(default=None)
 
     def __attrs_post_init__(self):
         if isinstance(self.service_equipment, str):
             object.__setattr__(self, "service_equipment", [self.service_equipment])
 
 
-@attr.s(auto_attribs=True)
+@define(auto_attribs=True)
 class Simulation(FromDictMixin):
     """The primary API to interact with the simulation methodologies.
 
     Parameters
     ----------
-    name: str
-        Name of the simulation. Used for logging files.
     library_path : str
         The path to the main data library.
-    config : Union[str, dict]
-        The path to a configuration dictionary or the dictionary itself.
+    config : Configuration | dict | str
+        One of the following:
+         - A pre-loaded ``Configuration`` object
+         - A dictionary ready to be converted to a ``Configuration`` object
+         - The name of the configuration file to be loaded, that will be located at:
+           ``library_path`` / config / ``config``
     """
 
-    name: str = attr.ib(converter=str)
-    library_path: Path = attr.ib(converter=_library_mapper)
-    config: Configuration = attr.ib()
+    library_path: Path = field(converter=_library_mapper)
+    config: Configuration = field()
+
+    metrics: Metrics = field(init=False)
+    windfarm: Windfarm = field(init=False)
+    env: WombatEnvironment = field(init=False)
+    repair_manager: RepairManager = field(init=False)
+    service_equipment: list[ServiceEquipment] = field(init=False)
 
     def __attrs_post_init__(self) -> None:
         self._setup_simulation()
 
     @config.validator  # type: ignore
     def _create_configuration(
-        self, attribute: attr.Attribute, value: Union[str, dict, Configuration]
+        self, attribute: Attribute, value: Union[str, dict, Configuration]
     ) -> None:
         """Validates the configuration object and creates the ``Configuration`` object
         for the simulation.Raises:
@@ -149,8 +159,6 @@ class Simulation(FromDictMixin):
                 "dictionary, or ``Configuration`` object!",
             )
 
-        if self.config.name != self.name:
-            raise ValueError("``name`` and the name in ``config`` do not match!")
         if self.config.library != self.library_path:
             raise ValueError(
                 "``library_path`` and the library in ``config`` do not match!"
@@ -194,7 +202,8 @@ class Simulation(FromDictMixin):
                 "``config`` must be a dictionary or ``Configuration`` object!"
             )
         assert isinstance(config, Configuration)  # lets mypy know it's a Configuration
-        return cls(config.name, config.library, config)
+        # NOTE: mypy is not caught up with attrs yet :(
+        return cls(config.library, config)  # type: ignore
 
     def _setup_simulation(self):
         """Initializes the simulation objects."""
@@ -217,7 +226,12 @@ class Simulation(FromDictMixin):
                 )
             )
 
-    def run(self, until: Optional[Union[int, float, Event]] = None):
+    def run(
+        self,
+        until: Optional[int | float | Event] = None,
+        create_metrics: bool = True,
+        save_metrics_inputs: bool = True,
+    ):
         """Calls ``WombatEnvironment.run()`` and gathers the results for post-processing.
         See ``wombat.simulation.WombatEnvironment.run`` or ``simpy.Environment.run`` for more
         details.
@@ -227,9 +241,21 @@ class Simulation(FromDictMixin):
         until : Optional[Union[int, float, Event]], optional
             When to stop the simulation, by default None. See documentation on
             ``simpy.Environment.run`` for more details.
+        create_metrics : bool, optional
+            If True, the metrics object will be created, and not, if False, by default True.
+        save_metrics_inputs : bool, optional
+            If True, the metrics inputs data will be saved to a yaml file, with file
+            references to any larger data structures that can be reloaded later. If
+            False, the data will not be saved, by default True.
         """
         self.env.run(until=until)
+        if save_metrics_inputs:
+            self.save_metrics_inputs()
+        if create_metrics:
+            self.initialize_metrics()
 
+    def initialize_metrics(self) -> None:
+        """Instantiates the ``metrics`` attribute after the simulation is run."""
         operations, events = self.env.convert_logs_to_csv(return_df=True)
         power_potential, power_production = self.env.power_production_potential_to_csv(
             windfarm=self.windfarm, operations=operations, return_df=True
@@ -243,8 +269,7 @@ class Simulation(FromDictMixin):
             inflation_rate=self.config.inflation_rate,
             project_capacity=self.config.project_capacity,
             turbine_capacities=[
-                self.windfarm.node_system(t_id).capacity
-                for t_id in self.windfarm.turbine_id
+                self.windfarm.system(t_id).capacity for t_id in self.windfarm.turbine_id
             ],
             fixed_costs=self.config.fixed_costs,
             substation_id=self.windfarm.substation_id.tolist(),
@@ -252,3 +277,37 @@ class Simulation(FromDictMixin):
             service_equipment_names=[el.settings.name for el in self.service_equipment],
             SAM_settings=self.config.SAM_settings,
         )
+
+    def save_metrics_inputs(self) -> None:
+        """Saves the inputs for the `Metrics` initialization with either a direct
+        copy of the data or a file reference that can be loaded later.
+        """
+        data = dict(
+            data_dir=str(self.config.library),
+            events=self.env.events_log_fname.replace(".log", ".csv"),
+            operations=self.env.operations_log_fname.replace(".log", ".csv"),
+            potential=self.env.power_potential_fname,
+            production=self.env.power_production_fname,
+            inflation_rate=self.config.inflation_rate,
+            project_capacity=self.config.project_capacity,
+            turbine_capacities=[
+                self.windfarm.system(t_id).capacity for t_id in self.windfarm.turbine_id
+            ],
+            fixed_costs=self.config.fixed_costs,
+            substation_id=self.windfarm.substation_id.tolist(),
+            turbine_id=self.windfarm.turbine_id.tolist(),
+            service_equipment_names=[el.settings.name for el in self.service_equipment],
+            SAM_settings=self.config.SAM_settings,
+        )
+
+        # Get the filename and rename it apporpriately
+        events_fname = Path(self.env.events_log_fname)
+        fname = events_fname.name
+        fname = fname.replace("_events.log", "_metrics_inputs.yaml")
+
+        # Get the path and move it 1 level up to <data_dir>/outputs/
+        fpath = events_fname.parents[1]
+
+        fname = fpath / fname  # type: ignore
+        with open(fname, "w") as f:
+            yaml.dump(data, f, default_flow_style=False, sort_keys=False)
